@@ -5,7 +5,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
+
 
 import atexit
 import os
@@ -27,11 +27,13 @@ from .qtcore import (
     QObject,
     QProcess,
     QSettings,
+    QSignalMapper,
     QSize,
     QUrl,
     QT_API,
     QT_VERSION,
     Qt,
+    pyqtBoundSignal,
     pyqtSignal,
     pyqtSlot,
 )
@@ -116,12 +118,8 @@ except ImportError:
 
 _W = TypeVar('_W', bound=QWidget)
 
-if pycompat.ispy3:
-    from html import escape as htmlescape
-else:
-    import cgi
-    def htmlescape(s: Text, quote: bool = True) -> Text:
-        return cgi.escape(s, quote)  # pytype: disable=module-attr
+# def htmlescape(s: Text, quote: bool = True) -> Text:
+from html import escape as htmlescape
 
 # largest allowed size for widget, defined in <src/gui/kernel/qwidget.h>
 QWIDGETSIZE_MAX = (1 << 24) - 1
@@ -201,7 +199,7 @@ def editfiles(repo, files, lineno=None, search=None, parent=None):
                              _('Please configure a visual editor.'))
         from tortoisehg.hgqt.settings import SettingsDialog
         dlg = SettingsDialog(False, focus='tortoisehg.editor')
-        dlg.exec_()
+        dlg.exec()
         return
 
     files = [procutil.shellquote(util.localpath(f)) for f in files]
@@ -289,7 +287,7 @@ def editfiles(repo, files, lineno=None, search=None, parent=None):
         QMessageBox.warning(parent,
                 _('Editor launch failure'),
                 u'%s : %s' % (hglib.tounicode(cmdline),
-                              hglib.tounicode(str(e))))
+                              hglib.exception_str(e)))
 
 def openshell(
     root: bytes,
@@ -343,6 +341,20 @@ def openshell(
                    _('A terminal shell must be configured'))
 
 
+def qbytearray_or_bytes_to_bytes(val: Union[QByteArray, bytes]) -> bytes:
+    if isinstance(val, bytes):
+        return cast(bytes, val)
+    else:
+        return val.data()
+
+
+def keyCombinationValue(combination) -> int:
+    if isinstance(combination, int):
+        return combination  # PyQt5
+    else:
+        return combination.toCombined()  # QKeyCombination
+
+
 # 'type' argument of QSettings.value() can't be used because:
 #  a) it appears to be broken before PyQt 4.11.x (#4882)
 #  b) it may raise TypeError if a setting has a value of an unexpected type
@@ -476,7 +488,6 @@ def configstyles(ui: uimod.ui) -> None:
 
 def geteffect(labels: Text) -> Text:
     'map labels like "log.date" to Qt font styles'
-    labels = str(labels) # Could be QString
     effects = []
     # Multiple labels may be requested
     for l in labels.split():
@@ -498,7 +509,7 @@ def geteffect(labels: Text) -> Text:
 
 def gettextcoloreffect(labels: Text) -> QColor:
     """Map labels like "log.date" to foreground color if available"""
-    for l in str(labels).split():
+    for l in labels.split():
         if not l:
             continue
         for e in _styles.get(l, '').split():
@@ -511,7 +522,7 @@ def getbgcoloreffect(labels: Text) -> QColor:
 
     Returns QColor object. You may need to check validity by isValid().
     """
-    for l in str(labels).split():
+    for l in labels.split():
         if not l:
             continue
         for e in _styles.get(l, '').split():
@@ -723,10 +734,12 @@ def geticon(name: Text) -> QIcon:
     try:
         return _iconcache[name]
     except KeyError:
-        _iconcache[name] = (_findthemeicon(name)
-                            or _findscalableicon(name)
-                            or _findcustomicon(name)
-                            or QIcon())
+        icon = (_findthemeicon(name)
+                or _findscalableicon(name)
+                or _findcustomicon(name)
+                or QIcon())
+        assert icon is not None  # help pytype
+        _iconcache[name] = icon
         return _iconcache[name]
 
 
@@ -854,7 +867,7 @@ def CommonMsgBox(
     title: Text,
     main: Text,
     text: Text = '',
-    buttons: Union[QMessageBox.StandardButtons, QMessageBox.StandardButton] = QMessageBox.StandardButton.Ok,
+    buttons: "Union[QMessageBox.StandardButtons, QMessageBox.StandardButton]" = QMessageBox.StandardButton.Ok,
     labels: Optional[Iterable[Tuple[QMessageBox.StandardButton, Text]]] = None,
     parent: Optional[QWidget] = None,
     defaultbutton: Optional[Union[QPushButton, QMessageBox.StandardButton]] = None,
@@ -874,7 +887,7 @@ def CommonMsgBox(
     for line in text.split('\n'):
         info += '<nobr>%s</nobr><br />' % line
     msg.setInformativeText(info)
-    return msg.exec_()
+    return msg.exec()
 
 def InfoMsgBox(*args: Text, **kargs) -> int:
     return CommonMsgBox(QMessageBox.Icon.Information, *args, **kargs)
@@ -926,7 +939,7 @@ class CustomPrompt(QMessageBox):
                 self.setEscapeButton(btn)
 
     def run(self) -> int:
-        return self.exec_()
+        return self.exec()
 
     def keyPressEvent(self, event: QKeyEvent) -> None:
         for k, btn in self.hotkeys.items():
@@ -973,7 +986,7 @@ class ChoicePrompt(QDialog):
 
     def run(self):
         # type: () -> Optional[Text]
-        if self.exec_():
+        if self.exec():
             return self.choices[self.choice_combo.currentIndex()]
         return None
 
@@ -1005,7 +1018,7 @@ class BadCompletionBlocker(QObject):
     >>> QApplication.sendEvent(
     ...     combo, QKeyEvent(QEvent.Type.KeyPress, Qt.Key.Key_Enter, Qt.KeyboardModifier.NoModifier))
     True
-    >>> str(combo.currentText())
+    >>> combo.currentText()
     'history value'
 
     In this example, QLineControl picks the first item in the combo box
@@ -1019,7 +1032,7 @@ class BadCompletionBlocker(QObject):
     >>> QApplication.sendEvent(
     ...     combo, QKeyEvent(QEvent.Type.KeyPress, Qt.Key.Key_Enter, Qt.KeyboardModifier.NoModifier))
     True
-    >>> str(combo.currentText())
+    >>> combo.currentText()
     'initial value'
 
     For details, read QLineControl::processKeyEvent() and complete() of
@@ -1320,7 +1333,7 @@ class DialogKeeper(QObject):
     >>> def processDeferredDeletion():
     ...     loop = QEventLoop()
     ...     QTimer.singleShot(0, loop.quit)
-    ...     loop.exec_()
+    ...     loop.exec()
 
     >>> dlg1.reject()
     >>> processDeferredDeletion()
@@ -1519,7 +1532,7 @@ def getCurrentUsername(widget, repo, opts=None):
                 QMessageBox.StandardButton.Ok)
     from tortoisehg.hgqt.settings import SettingsDialog
     dlg = SettingsDialog(False, focus='ui.username')
-    dlg.exec_()
+    dlg.exec()
     repo.invalidateui()
     return hglib.tounicode(hglib.configuredusername(repo.ui))
 
@@ -1539,7 +1552,7 @@ def getTextInput(
     label: Text,
     mode: QLineEdit.EchoMode = QLineEdit.EchoMode.Normal,
     text: Text = '',
-    flags: Union[Qt.WindowType, Qt.WindowFlags] = Qt.WindowType.Widget,
+    flags: "Union[Qt.WindowType, Qt.WindowFlags]" = Qt.WindowType.Widget,
 ) -> Tuple[Text, bool]:
     flags |= (Qt.WindowType.CustomizeWindowHint | Qt.WindowType.WindowTitleHint
               | Qt.WindowType.WindowCloseButtonHint)
@@ -1549,7 +1562,7 @@ def getTextInput(
     dlg.setTextValue(text)
     dlg.setTextEchoMode(mode)
 
-    r = dlg.exec_()
+    r = dlg.exec()
     dlg.setParent(None)  # so that garbage collected
     return r and dlg.textValue() or '', bool(r)
 
@@ -1628,3 +1641,20 @@ def setContextMenuShortcuts(action, shortcuts):
     action.setShortcuts(shortcuts)
     if QT_VERSION >= 0x50a00 and PYQT_VERSION >= 0x50a00:
         action.setShortcutVisibleInContextMenu(True)
+
+def getMappedStringSignal(signalMapper: QSignalMapper) -> pyqtBoundSignal:
+    try:
+        return signalMapper.mappedString[str]
+    except AttributeError:
+        # qt < 5.15
+        return signalMapper.mapped[str]
+
+def toCheckStateEnum(value: Union[int, Qt.CheckState]) -> Qt.CheckState:
+    """Coerce a Qt.CheckState value to the corresponding enum value."""
+    if PYQT_VERSION >= 0x060000:
+        if isinstance(value, int):
+            # pytype: disable=attribute-error
+            return next(e for e in Qt.CheckState if e.value == value)
+            # pytype: enable=attribute-error
+
+    return cast(Qt.CheckState, value)
